CON _clkmode = xtal1 + pll16x
    _xinfreq = 5_000_000       '80 MHz

    PAGESIZE = 32
    
VAR byte pauseval
    byte buffer[PAGESIZE*2]
    
OBJ
    SD  : "FSRW"
    sdspi : "sdspiFemto"       ' SPIN program loader and support routines  
    i2c : "Basic_I2C_Driver" 

PUB start|i,j 
  light(5)
  
  'access SD card 
  i:=sd.mount(0)
  if (i<>0)
    loadTop 'No SD Card load top half

  light(6)
  
  'See if code.eeprom was present
  i:=sd.popen(string("code.bin"), "r")
  if (i<>0)
    loadTop 'File not there so load top half

  light(7) 
    
  'Load EEPROM code from uSD card
  repeat j from 0 to constant(($8000/PAGESIZE)-1)
    i:=sd.pread(@buffer, PAGESIZE)
    i2c.WritePage(i2c#BootPin, i2c#EEPROM, $8000+(j*PAGESIZE), @buffer, PAGESIZE)

  light(8)
    
  'if nodelete.txt is not present delete image
  if (sd.popen(string("nodelete.txt"), "r")<>0)
    sd.popen(string("code.bin"), "d" )

  light(9)
    
  'load the EEPROM contents
  loadTop
PUB light(num)
  dira:=$0fff_fff0
  outa:=1<<num
  waitcnt(cnt+clkfreq )  
  waitcnt(cnt+clkfreq )
  
PRI loadTop
  light(10)

  sdspi.bootEEPROM(sdspi#bootAddr + $8000)

DAT
  ORG 0
        'repeat j from 0 to constant((32768/PAGESIZE)-1) 
ASMCOPY MOV   jj,pages                                   
 
        'ReadPage(i2c#EEPROM, 32768+j*PAGESIZE, j*PAGESIZE, PAGESIZE)  
:loop   MOV   devSel,$A0
        MOV   addrReg,eepromp
        MOV   dataPtr,romp
        MOV   count,#32                                 '<------------------- PAGESIZE    
        CALL  #readp
        ADD   eepromp,#32                               '<------------------- PAGESIZE
        ADD   romp,#32                                  '<------------------- PAGESIZE

        '}
        DJNZ  jj,#:loop
        
        'restart the prop
        cogid  self
        coginit interpreter     'start Spin Interpreter in Cog 0
        
        'stop this cog
        cogstop self             'and stop this ASM cog

interpreter     long    ($0004 << 16) | ($F004 << 2) | %0000
self    res 1

pages   LONG  1023                                      '<------------------- (32768/PAGESIZE)-1
eepromp LONG  32768                                     'start of top half of eeprom
romp    LONG  0                                         'start of ram
'************************************************************************************
   
{{to use:
        MOV   devSel,$A0
        MOV   addrReg,eeprom_start_location
        MOV   dataPtr,hubram_start_location
        MOV   count,PAGESIZE      
        CALL  readp
}}        
   'devSel |= addrReg >> 15 & %1110
readp   MOV   temp,addrReg
        SHR   temp,#15
        AND   temp,%1110
        OR    devSel,temp
  
  'Start(SCL)                          ' Select the device & send address
        CALL  #starti2c
        
  'ackbit := Write(SCL, devSel | Xmit)
        MOV   data,devSel
        OR    data,Xmit
        CALL  #write
        MOV   ackbits,ackbit
        
  'ackbit := (ackbit << 1) | Write(SCL, addrReg >> 8 & $FF)
        MOV   data,addrReg
        SHR   data,#8
        AND   data,#$FF
        CALL  #write
        SHL   ackbits,#1
        OR    ackbits,ackbit
  
  'ackbit := (ackbit << 1) | Write(SCL, addrReg & $FF)
        MOV   data,addrReg
        AND   data,#$FF
        CALL  #write
        SHL   ackbits,#1
        OR    ackbits,ackbit
        
  'Start(SCL)                          ' Reselect the device for reading
        CALL  #starti2c
        
  'ackbit := (ackbit << 1) | Write(SCL, devSel | Recv)
        MOV   data,devSel
        OR    data,Recv
        CALL  #write
        SHL   ackbits,#1
        OR    ackbits,ackbit
  
  'repeat count - 1
        SUB   count,#1
     
  '   byte[dataPtr++] := Read(SCL, ACK)
:loop   MOV   ackbit,ACK
        CALL  #read
        WRBYTE data,dataPtr
        ADD   dataPtr,#1                      

  '}
        DJNZ  count,#:loop
          
  'byte[dataPtr++] := Read(SCL, NAK)
        MOV   ackbit,NAK
        CALL  #read
        WRBYTE data,dataPtr
        ADD   dataPtr,#1
        
  'Stop(SCL)
        CALL  #stop
        
  'return ackbit
readp_ret     RET

devSel  LONG  0
Xmit    LONG  0
Recv    LONG  1
ACK     LONG  0
NAK     LONG  1
ackbits LONG  0
addrReg LONG  0
count   LONG  0
dataPtr LONG  0
'************************************************************************************
   
{{to use:
        CALL  start
}}
        'outa[SCL]~~                         ' Initially drive SCL HIGH
starti2c OR    OUTA,SCLbit
          
        'dira[SCL]~~
        OR    DIRA,SCLbit

        'outa[SDA]~~                         ' Initially drive SDA HIGH
        OR    OUTA,SDAbit

        'dira[SDA]~~
        OR    DIRA,SDAbit
        
        'outa[SDA]~                          ' Now drive SDA LOW
        AND   OUTA,SDAmask

        'outa[SCL]~                          ' Leave SCL LOW
        AND   OUTA,SCLmask
starti2c_ret  RET

'************************************************************************************

{{to use:
        CALL  stop
}}      'outa[SCL]~~                         ' Drive SCL HIGH
stop    OR    OUTA,SCLbit
        
        'outa[SDA]~~                         '  then SDA HIGH
        OR    OUTA,SDAbit
        
        'dira[SCL]~                          ' Now let them float
        AND   DIRA,SCLmask
        
        'dira[SDA]~                          ' If pullups present, they'll stay HIGH
        AND   DIRA,SDAmask
stop_ret      RET

'************************************************************************************

{{to use:
        MOV   data,??????
        CALL  write
        MOV   ????,ackbit
}}      'data <<= 24
write   SHL   data,#24
        
        'repeat 8                            ' Output data to SDA
        MOV   temp,#8

          'outa[SDA] := (data <-= 1) & 1
:loop   ROL   data,#1
        TEST  data,#1   WC
        MUXC  OUTA,SDAbit
           
          'outa[SCL]~~                       ' Toggle SCL from LOW to HIGH to LOW
        OR    OUTA,SCLbit
        
          'outa[SCL]~
        AND   OUTA,SCLmask

        '}
        DJNZ  temp,#:loop        
        
        'dira[SDA]~                          ' Set SDA to input for ACK/NAK
        AND   DIRA,SDAmask
        
        'outa[SCL]~~
        OR    OUTA,SCLbit
        
        'ackbit := ina[SDA]                  ' Sample SDA when SCL is HIGH
        TEST  INA,SDAbit        WZ
        MUXNZ ackbit,#1
        
        'outa[SCL]~
        AND   OUTA,SCLmask
        
        'outa[SDA]~                          ' Leave SDA driven LOW
        AND   OUTA,SDAmask
        
        'dira[SDA]~~
        OR    DIRA,SDAbit
write_ret     RET        

'************************************************************************************

{{to use:
        MOV   ackbit,??????
        CALL  read
        MOV   ????,data
}}'PUB Read(SCL, ackbit): data | SDA
        'data := 0
        MOV   data,#0

        'dira[SDA]~                          ' Make SDA an input
read    AND   DIRA,SDAmask
 
        'repeat 8                            ' Receive data from SDA
        MOV   temp,#8
        
          'outa[SCL]~~                      ' Sample SDA when SCL is HIGH
:loop   OR    OUTA,SCLbit
        
          'data := (data << 1) | ina[SDA]
        SHL   data,#1
        TEST  INA,SDAbit        WZ
        MUXNZ data,#1
        
          'outa[SCL]~
        AND   OUTA,SCLmask

        '}
        DJNZ  temp,#:loop
        
        'outa[SDA] := ackbit                 ' Output ACK/NAK to SDA
        TEST  ackbit,#1         WZ
        MUXNZ OUTA,SDAbit
        
        'dira[SDA]~~
        OR    DIRA,SDAbit
        
        'outa[SCL]~~                         ' Toggle SCL from LOW to HIGH to LOW
        OR    DIRA,SCLbit
        
        'outa[SCL]~
        AND   OUTA,SCLmask
        
        'outa[SDA]~                          ' Leave SDA driven LOW
        AND   OUTA,SDAmask
read_ret      RET


SCLbit  LONG  %0001_0000_0000_0000_0000_0000_0000_0000
SDAbit  LONG  %0010_0000_0000_0000_0000_0000_0000_0000
SCLmask LONG  %1110_1111_1111_1111_1111_1111_1111_1111
SDAmask LONG  %1101_1111_1111_1111_1111_1111_1111_1111
data    LONG  0
temp    LONG  0
ackbit  LONG  0
jj      LONG  0         

        FIT




{{
                            TERMS OF USE: MIT License

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
}}